

local init_inst , init_inst_index
local init_sample_index , init_sample , init_buffer

local init_sample_rate , init_bit_depth
local init_number_of_channels , init_number_of_frames
local init_sync_enabled , init_sync_lines
local init_base_note , init_fine_tune

local new_sample , new_buffer

local bpm, lpb, selection_start, selection_end



-- init sample settings

function get_init_env()
  init_inst = renoise.song().selected_instrument
  init_sample_index = renoise.song().selected_sample_index
  init_sample = renoise.song().selected_sample
  init_buffer = renoise.song().selected_sample.sample_buffer
  
  if init_buffer.has_sample_data ~= true
    then
      renoise.app():show_error(" No sample data.")
    return nil
  end
  
  init_sample_rate =
       init_buffer.sample_rate
  init_bit_depth =
       init_buffer.bit_depth
  init_number_of_channels =
  init_buffer.number_of_channels
  init_number_of_frames =
  init_buffer.number_of_frames
  
  init_sync_enabled = init_sample.beat_sync_enabled
  init_sync_lines = init_sample.beat_sync_lines
  
  init_base_note = init_sample.base_note
  init_fine_tune = init_sample.fine_tune

  
  
  selection_start =
          init_buffer.selection_start
  selection_end =
          init_buffer.selection_end
  bpm = renoise.song().transport.bpm
  lpb = renoise.song().transport.lpb
  
end



-- new sample settings

function get_new_env()
  init_inst:insert_sample_at(init_sample_index +1)
  new_sample =
          init_inst.samples[init_sample_index +1]
          
  new_sample.beat_sync_enabled =  init_sync_enabled
  --  new_sample.beat_sync_lines =  init_sync_lines

  new_sample.base_note = init_base_note
  new_sample.fine_tune = init_fine_tune



  new_buffer = new_sample.sample_buffer
  
end



-- http://lua-users.org/wiki/SimpleRound
function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end


function beat_unit_with_base_tune ()
--  get_init_env()
  local bn = init_base_note
  local ft = init_fine_tune

  if
    (bn == 48 and ft == 0) or
    init_sync_enabled == true
  then return 1
  else return
  math.pow ((1/2),((bn-48)-(ft/128))/12)
  end
end


function beat_unit_with_sync ()
--   get_init_env()
           
   if init_sync_enabled == false
   then return 1
   else
     return
     (init_number_of_frames * (lpb / init_sync_lines))/
     ((1 / bpm * 60) * init_sample_rate)
   end
end


function beat_org (beat)
  get_init_env()
  return
  beat * beat_unit_with_sync () * beat_unit_with_base_tune ()
end



function frames_per_xbeat (beat)
  bpm = renoise.song().transport.bpm
  local seconds_per_beat = 1 / bpm * 60
  return round((seconds_per_beat * init_sample_rate * beat))
end



function selection2beat ()
   get_init_env()
   local val
     val = ((selection_end - selection_start +1) /
     ((1 / bpm * 60) * init_sample_rate)) /
      (beat_unit_with_sync () * beat_unit_with_base_tune ())
   return val
end





function upbeat_start_point(beat)
  selection_start =
          init_buffer.selection_start
  local s = (selection_start -1)
            - frames_per_xbeat(beat)
  if s >= 0 then
  return s+1
  else
  return s
  end
end


function frames_with_upbeat(upbeat)
  selection_start =
          init_buffer.selection_start
  selection_end =
          init_buffer.selection_end
  return
  frames_per_xbeat (upbeat)
  + selection_end
  - selection_start + 1
end


function create_new (num_frames)
--  get_init_env()
--  init_inst:insert_sample_at(init_sample_index+1)
  get_new_env()

  local new_sync_val =
          init_sync_lines * (num_frames/init_number_of_frames) 
  if init_sync_enabled == false or new_sync_val > 256 
    then
      new_sample.beat_sync_lines = init_sync_lines
    else
      new_sample.beat_sync_lines = new_sync_val
  end 

  new_buffer:create_sample_data(
                 init_sample_rate,init_bit_depth,init_number_of_channels
                 , num_frames)
end



function create_with_upbeat (beat)
  local bool =
  create_new (
    frames_with_upbeat(beat))
end



function set_new_with_upbeat (beat)
  get_init_env()

  
  create_with_upbeat (beat)
  
  local s = upbeat_start_point(beat)
  local new_num_frames = frames_with_upbeat(beat)
  

 new_buffer:prepare_sample_data_changes()
  
  if s > 0
    then          
      for ch = 1,init_number_of_channels do
        for fr = 1,new_num_frames do
          new_buffer
          :set_sample_data(ch,fr,
           init_buffer:sample_data(ch, fr+s-1))
        end
      end
    else
      
      for ch = 1,init_number_of_channels do
        for fr = 1,(-s) do
          new_buffer:set_sample_data(
            ch,fr,
            0)
        end
        for fr = (1-s),new_num_frames do
          new_buffer
          :set_sample_data(ch,fr,
          init_buffer
          :sample_data(ch,fr+s))
        end
      end
    end
 new_buffer:finalize_sample_data_changes()
end




function replace_with_upbeat (beat)
--  get_init_env()
  local beat_org = beat * beat_unit_with_sync ()
  
  set_new_with_upbeat (beat)
  init_sample:copy_from(new_sample)
  init_inst:delete_sample_at(init_sample_index + 1)
  init_buffer.selection_start = frames_per_xbeat(beat)+1
end


function set_end_point (beat)
  get_init_env()

  local bt_frame = frames_per_xbeat (beat)
  local end_point = selection_start + bt_frame -1

  if beat <= 0
    then renoise.app():show_error('Enter the number greater than zero')
  elseif
    end_point <= init_number_of_frames
    then
    init_buffer.selection_end  = end_point
  elseif
    end_point > init_number_of_frames
    then
    create_new (end_point)

    new_buffer:prepare_sample_data_changes()

      for ch = 1,init_number_of_channels do
        for fr = 1,init_number_of_frames do
          new_buffer
          :set_sample_data(
            ch,fr,
            init_buffer
            :sample_data(ch,fr))
        end
        for fr = init_number_of_frames+1,end_point do
          new_buffer
          :set_sample_data(
            ch,fr,0)
        end
      end      
    new_buffer:finalize_sample_data_changes()

      init_sample:copy_from(new_sample)
      init_inst:delete_sample_at(init_sample_index + 1)
      init_buffer.selection_start = selection_start

  else return nil
  end
end



function sweep_ins()
  get_init_env()
  local range = selection_end - selection_start +1
  if range == init_number_of_frames
  then return nil
  else
    local end_point = init_number_of_frames + range
    
    create_new (end_point)
      
    new_buffer:prepare_sample_data_changes()
    
        for ch = 1,init_number_of_channels do
          for fr = 1,selection_start -1 do
            new_buffer
            :set_sample_data(
              ch,fr,
              init_buffer
              :sample_data(ch,fr))
          end
          for fr = selection_start ,selection_end do
            new_buffer
            :set_sample_data(
              ch,fr,0)
          end
          for fr = selection_end +1,end_point do
            new_buffer
            :set_sample_data(
              ch,fr,
              init_buffer
              :sample_data(ch,fr-range))
          end        
        end      
      new_buffer:finalize_sample_data_changes()
    
        init_sample:copy_from(new_sample)
        init_inst:delete_sample_at(init_sample_index + 1)
        init_buffer.selection_start = selection_start
        init_buffer.selection_end = selection_end
  end
end


function erace()
  get_init_env()
  local range = selection_end - selection_start +1
  if range == init_number_of_frames
  then return nil
  else
    
      create_new (init_number_of_frames)
      
      new_buffer:prepare_sample_data_changes()
    
          for ch = 1,init_number_of_channels do
            for fr = 1,selection_start -1 do
              new_buffer
              :set_sample_data(
                ch,fr,
                init_buffer
                :sample_data(ch,fr))
            end
            for fr = selection_start ,selection_end do
              new_buffer
              :set_sample_data(
                ch,fr,0)
            end
            for fr = selection_end +1,init_number_of_frames do
              new_buffer
              :set_sample_data(
                ch,fr,
                init_buffer
                :sample_data(ch,fr))
            end        
          end      
        new_buffer:finalize_sample_data_changes()
    
          init_sample:copy_from(new_sample)
          init_inst:delete_sample_at(init_sample_index + 1)
          init_buffer.selection_start = selection_start
          init_buffer.selection_end = selection_end
  end
end


function sync_del()
  get_init_env()
  local range = selection_end - selection_start +1
  if range >= init_number_of_frames
  then return nil
  else
      local end_point = init_number_of_frames - range
    
      create_new (end_point)
      
      new_buffer:prepare_sample_data_changes()
    
          for ch = 1,init_number_of_channels do
            for fr = 1,selection_start -1 do
              new_buffer
              :set_sample_data(
                ch,fr,
                init_buffer
                :sample_data(ch,fr))
            end
            for fr = selection_start,end_point do
              new_buffer
              :set_sample_data(
                ch,fr,
                init_buffer
                :sample_data(ch,fr+range))
            end        
          end      
        new_buffer:finalize_sample_data_changes()
    
          init_sample:copy_from(new_sample)
          init_inst:delete_sample_at(init_sample_index + 1)
          init_buffer.selection_start = selection_start
          init_buffer.selection_end = selection_start +1
  end
end

       



